unit tools_stringTable;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, strutils, Graphics;

type tool_stringTable = class(TObject)
  private
    procedure calculate_and_draw;
  public
    constructor Create; overload;
    procedure clear();
    procedure loadFromString(strData: string);
    function getAsString() : string;
    procedure beginRow();
    procedure addCell(cellValue: string);
    procedure addCellInfo(cellInfoName: string; cellInfoValue: string);
    procedure endRow();
    function getCellInfoValue(cellX: Integer; cellY: Integer; infoName: string) : string;
    function getCellAsString(cellX: Integer; cellY: Integer) : string;
    function getRowCount() : Integer;
    function getColCount() : Integer;
    procedure canvas_draw(canvas: TCanvas; posX: Integer; posY: Integer; posW: Integer; posH: Integer);
    procedure free();
  var
    cellData:array[0..500,0..500] of string;
    rows:Integer;
    colsInRow:Integer;
    cols:Integer;

    calc_type:Integer; 
    calc_colsWidths:array[0..500] of integer;
    calc_posX:Integer;
    calc_posY:Integer;
    calc_posW:Integer;
    calc_posH:Integer;

    canvas_obj: TCanvas;
end;

implementation

constructor tool_stringTable.Create;
begin
  inherited;
  clear;
end;

procedure tool_stringTable.clear();
begin
  colsInRow := 0;
  rows := 0;
  cols := 0;
end;

procedure tool_stringTable.loadFromString(strData: string);
var
  dataRows: TStrings;
  dataColValues: TStrings;
  iRow: Integer;
  iCol: Integer;
begin
  clear();

  dataRows := TStringList.Create;
  try
    ExtractStrings([chr(10)], [], PChar(strData), dataRows);
    rows := dataRows.Count;
    for iRow := 0 to dataRows.Count - 1 do
    begin
      dataColValues := TStringList.Create;

      try
        ExtractStrings([','], [], PChar( dataRows.Strings[iRow] ), dataColValues);
        if dataColValues.Count > cols then
        begin
          cols := colsInRow;
        end;
        for iCol := 0 to dataColValues.Count - 1 do
        begin
          cellData[iRow, iCol] := dataColValues.Strings[iCol];
        end;
      finally
        dataColValues.Free;
      end;
    end;
  finally
    dataRows.Free;
  end;

end;

function tool_stringTable.getAsString() : string;
var
  tableS   : string;
  tableNlS : string;
  iRow     : Integer;
  iCol     : Integer;
begin
  tableNlS := chr(10);
  tableS := '';
  for iRow := 0 to rows - 1 do
  begin
       for iCol := 0 to cols - 2 do
       begin
         tableS := tableS + cellData[iCol, iRow] + ',';
       end;
       tableS := tableS + cellData[cols - 1, iRow];
       tableS := tableS + tableNlS;
  end;

  getAsString := tableS;
end;

procedure tool_stringTable.beginRow();
begin
  colsInRow := 0;
  rows := rows + 1;
  
end;

procedure tool_stringTable.addCell(cellValue: string);
begin
  colsInRow := colsInRow + 1;
  if colsInRow > cols then
  begin
    cols := colsInRow;
  end;
  
  cellValue := StringReplace(cellValue, ',', '[as3jD]', []);
  cellData[rows - 1, colsInRow - 1] := cellValue;
end;

procedure tool_stringTable.addCellInfo(cellInfoName: string; cellInfoValue: string);
begin
  cellData[rows - 1, colsInRow - 1] := cellData[rows - 1, colsInRow - 1] + '[sd72j]' + cellInfoName + '[=]' + cellInfoValue + '[et]';
end;

procedure tool_stringTable.endRow();
begin
  // ...
end;

function tool_stringTable.getCellAsString(cellX: Integer; cellY: Integer) : string;
var
  wrkVal: string;
  retVal: string;
  po1: Integer;
begin
  retVal := '';
  wrkVal := cellData[cellX, cellY];
  po1 := NPos('[sd72j]', wrkVal, 1);
  if po1 > -1 then
  begin
    wrkVal := LeftStr(wrkVal, po1 - 1);
    retVal := wrkVal;
    retVal := StringReplace(retVal, '[as3jD]', ',', []);
  end;
  if po1 = -1 then
  begin
    retVal := StringReplace(wrkVal, '[as3jD]', ',', []);
  end;
  getCellAsString := retVal
end;

function tool_stringTable.getCellInfoValue(cellX: Integer; cellY: Integer; infoName: string) : string;
var
  wrkVal: string;
  retVal: string;
  po1: Integer;
  po2: Integer;
begin
  retVal := '';
  wrkVal := cellData[cellX, cellY];
  po1 := NPos('[sd72j]' + infoName + '[=]', wrkVal, 1);
  if po1 > -1 then
  begin
    po1 := po1 + 7 + Length(infoName) + 3;
    po2 := NPos('[et]', wrkVal, 1);
    if po2 > -1 then
    begin
      wrkVal := Copy(wrkVal, po1, po2 - po1);
      retVal := wrkVal;
      retVal := StringReplace(retVal, '[as3jD]', ',', []);
    end;
    {
    wrkVal := Copy(wrkVal, po1, Length(wrkVal) - po1 - 3);
    retVal := wrkVal;
    retVal := StringReplace(retVal, '[as3jD]', ',', []);
    }
  end;
  getCellInfoValue := retVal;
end;

function tool_stringTable.getRowCount() : Integer;
begin
  getRowCount := cols;
end;

function tool_stringTable.getColCount() : Integer;
begin
  getColCount := rows;
end;

procedure tool_stringTable.calculate_and_draw;
var
   iRow : Integer;
   iCol : Integer;

   colWidth: Integer;
   colHeight: Integer;
   rowHeight: Integer;
   colOutputContent: string;

   posX: Integer;
   posY: Integer;
begin
  for iCol := 0 to cols - 2 do
  begin
    calc_colsWidths[iCol] := 0;
  end;
  cellData[0, 0] := '!!!';

  if calc_type = 0 then
  begin
    for iRow := 0 to rows - 1 do
    begin
      for iCol := 0 to cols - 1 do
      begin
        colOutputContent := getCellAsString(iCol, iRow);
        colWidth := canvas_obj.TextWidth(colOutputContent);
        if colWidth > calc_colsWidths[iCol] then
        begin
          calc_colsWidths[iCol] := colWidth;
        end;
      end;
    end;

    posX := 0;
    posY := 0;
    for iRow := 0 to rows - 1 do
    begin
      rowHeight := 0;
      for iCol := 0 to cols - 1 do
      begin
        colOutputContent := getCellAsString(iCol, iRow);
        colHeight := canvas_obj.TextHeight(colOutputContent);
        if colHeight > rowHeight then
        begin
          rowHeight := colHeight;
        end;
      end;

      for iCol := 0 to cols - 1 do
      begin
        colOutputContent := getCellAsString(iCol, iRow);
        canvas_obj.TextRect( Rect(posX, posY, posX+calc_colsWidths[iCol], posY+rowHeight), 0, 0, colOutputContent );

        posX := posX + calc_colsWidths[iCol];
      end;

      posY := posY + rowHeight;
    end;
  end;
end;

procedure tool_stringTable.canvas_draw(canvas: TCanvas; posX: Integer; posY: Integer; posW: Integer; posH: Integer);
begin
  calc_type := 0;
  calc_posX := posX;
  calc_posY := posY;
  calc_posW := posW;
  calc_posH := posH;
  canvas_obj := canvas;
  calculate_and_draw;
end;

procedure tool_stringTable.free();
begin
  // ...
end;


end.

